@@ -20,7 +20,9 @@ and cache hits NOT refreshing the TTL of cached values.
 Set the environment variables QSV_REDIS_CONNSTR, QSV_REDIS_TTL_SECONDS and 
 QSV_REDIS_TTL_REFRESH to change default Redis settings.
 
-Supports brotli, gzip and deflate compression for improved throughput & performance.
+Supports brotli, gzip and deflate automatic decompression for improved throughput and
+performance, preferring brotli over gzip over deflate.
+
 Automatically upgrades its connection to HTTP/2 as well if the server supports it.
 (see https://www.cloudflare.com/learning/performance/http2-vs-http1.1/)
 
