@@ -1,46 +1,15 @@
-#![allow(dead_code)]
 extern crate crossbeam_channel as channel;
-
-use std::borrow::ToOwned;
-use std::env;
-use std::fmt;
-use std::io;
-use std::process;
-use std::time::Instant;
+use std::{env, io, time::Instant};
 
 use docopt::Docopt;
-use log::{error, info, log_enabled, Level};
 use serde::Deserialize;
 
+use crate::clitypes::{CliError, CliResult, QsvExitCode};
+
 #[cfg(feature = "mimalloc")]
 #[global_allocator]
 static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
 
-macro_rules! wout {
-    ($($arg:tt)*) => ({
-        use std::io::Write;
-        (writeln!(&mut ::std::io::stdout(), $($arg)*)).unwrap();
-    });
-}
-
-macro_rules! werr {
-    ($($arg:tt)*) => ({
-        use std::io::Write;
-        use log::error;
-        error!("{}", $($arg)*);
-        (writeln!(&mut ::std::io::stderr(), $($arg)*)).unwrap();
-    });
-}
-
-macro_rules! fail {
-    ($e:expr) => {{
-        use log::error;
-        let err = ::std::convert::From::from($e);
-        error!("{err}");
-        Err(err)
-    }};
-}
-
 macro_rules! command_list {
     () => {
         "
@@ -69,6 +38,7 @@ macro_rules! command_list {
     rename      Rename the columns of CSV data efficiently
     replace     Replace patterns in CSV data
     reverse     Reverse rows of CSV data
+    safenames   Modify a CSV's header names to db-safe names
     sample      Randomly sample CSV data
     schema      Generate JSON Schema from CSV data
     search      Search CSV data with a regex
@@ -77,9 +47,11 @@ macro_rules! command_list {
     slice       Slice records from CSV
     sniff       Quickly sniff CSV metadata
     sort        Sort CSV data in alphabetical, numerical, reverse or random order
+    sortcheck   Check if a CSV is sorted
     split       Split CSV data into many files
-    stats       Infer data types and compute descriptive statistics
+    stats       Infer data types and compute summary statistics
     table       Align CSV data into columns
+    tojsonl     Convert CSV to newline-delimited JSON
     transpose   Transpose rows/columns of CSV data
     validate    Validate CSV data for RFC4180-compliance or with JSON Schema
 
@@ -87,14 +59,14 @@ macro_rules! command_list {
 "
     };
 }
+mod clitypes;
 mod cmd;
 mod config;
 mod index;
 mod select;
 mod util;
 
-static USAGE: &str = concat!(
-    "
+static USAGE: &str = r#"
 Usage:
     qsvlite <command> [<args>...]
     qsvlite [options]
@@ -103,33 +75,27 @@ Options:
     --list               List all commands available.
     --envlist            List all qsv-relevant environment variables.
     -u, --update         Update qsv to the latest release from GitHub.
+    -U, --updatenow      Update qsv to the latest release from GitHub without confirming.
     -h, --help           Display this message
     <command> -h         Display the command help message
     -v, --version        Print version info, mem allocator, features installed, 
-                         max_jobs, num_cpus then exit
+                         max_jobs, num_cpus, build info then exit
 
 * sponsored by datHere - Data Infrastructure Engineering
-"
-);
+"#;
 
 #[derive(Deserialize)]
 struct Args {
-    arg_command: Option<Command>,
-    flag_list: bool,
-    flag_envlist: bool,
-    flag_update: bool,
+    arg_command:    Option<Command>,
+    flag_list:      bool,
+    flag_envlist:   bool,
+    flag_update:    bool,
+    flag_updatenow: bool,
 }
 
-fn main() {
-    util::init_logger();
-
+fn main() -> QsvExitCode {
     let now = Instant::now();
-    let qsv_args: String = if log_enabled!(Level::Info) {
-        env::args().skip(1).collect::<Vec<_>>().join(" ")
-    } else {
-        "".to_string()
-    };
-    info!("START: {qsv_args}");
+    let qsv_args = util::init_logger();
 
     let args: Args = Docopt::new(USAGE)
         .and_then(|d| {
@@ -140,17 +106,20 @@ fn main() {
         .unwrap_or_else(|e| e.exit());
     if args.flag_list {
         wout!(concat!("Installed commands:", command_list!()));
-        log_end(qsv_args, &now);
-        return;
+        util::log_end(qsv_args, now);
+        return QsvExitCode::Good;
     } else if args.flag_envlist {
         util::show_env_vars();
-        log_end(qsv_args, &now);
-        return;
+        util::log_end(qsv_args, now);
+        return QsvExitCode::Good;
     }
-    if args.flag_update {
-        util::qsv_check_for_update();
-        log_end(qsv_args, &now);
-        return;
+    if args.flag_update || args.flag_updatenow {
+        let update_checked = util::qsv_check_for_update(false, args.flag_updatenow);
+        util::log_end(qsv_args, now);
+        if update_checked.is_ok() {
+            return QsvExitCode::Good;
+        }
+        return QsvExitCode::Bad;
     }
     match args.arg_command {
         None => {
@@ -160,54 +129,48 @@ fn main() {
 Please choose one of the following commands:",
                 command_list!()
             ));
-            util::qsv_check_for_update();
-            log_end(qsv_args, &now);
-            ::std::process::exit(0);
+            _ = util::qsv_check_for_update(true, false);
+            util::log_end(qsv_args, now);
+            QsvExitCode::Good
         }
         Some(cmd) => match cmd.run() {
             Ok(()) => {
-                log_end(qsv_args, &now);
-                process::exit(0);
+                util::log_end(qsv_args, now);
+                QsvExitCode::Good
+            }
+            Err(CliError::Flag(err)) => {
+                werr!("{err}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::IncorrectUsage
             }
-            Err(CliError::Flag(err)) => err.exit(),
             Err(CliError::Csv(err)) => {
                 werr!("{err}");
-                log_end(qsv_args, &now);
-                process::exit(1);
+                util::log_end(qsv_args, now);
+                QsvExitCode::Bad
             }
             Err(CliError::Io(ref err)) if err.kind() == io::ErrorKind::BrokenPipe => {
-                log_end(qsv_args, &now);
-                process::exit(0);
+                werr!("Broken pipe: {err}");
+                util::log_end(qsv_args, now);
+                QsvExitCode::Abort
             }
             Err(CliError::Io(err)) => {
                 werr!("{err}");
-                log_end(qsv_args, &now);
-                process::exit(1);
+                util::log_end(qsv_args, now);
+                QsvExitCode::Bad
+            }
+            Err(CliError::NoMatch()) => {
+                util::log_end(qsv_args, now);
+                QsvExitCode::Bad
             }
             Err(CliError::Other(msg)) => {
                 werr!("{msg}");
-                log_end(qsv_args, &now);
-                process::exit(1);
+                util::log_end(qsv_args, now);
+                QsvExitCode::Bad
             }
         },
     }
 }
 
-fn log_end(mut qsv_args: String, now: &Instant) {
-    if log_enabled!(Level::Info) {
-        let ellipsis = if qsv_args.len() > 24 {
-            qsv_args.truncate(24);
-            "..."
-        } else {
-            ""
-        };
-        info!(
-            "END \"{qsv_args}{ellipsis}\" elapsed: {}",
-            now.elapsed().as_secs_f32()
-        );
-    }
-}
-
 #[derive(Debug, Deserialize)]
 #[serde(rename_all = "lowercase")]
 enum Command {
@@ -236,6 +199,7 @@ enum Command {
     Rename,
     Replace,
     Reverse,
+    Safenames,
     Sample,
     Schema,
     Search,
@@ -244,9 +208,11 @@ enum Command {
     Slice,
     Sniff,
     Sort,
+    SortCheck,
     Split,
     Stats,
     Table,
+    Tojsonl,
     Transpose,
     Validate,
 }
@@ -281,7 +247,7 @@ impl Command {
             Command::Headers => cmd::headers::run(argv),
             Command::Help => {
                 wout!("{USAGE}");
-                util::qsv_check_for_update();
+                _ = util::qsv_check_for_update(true, false);
                 Ok(())
             }
             Command::Index => cmd::index::run(argv),
@@ -293,6 +259,7 @@ impl Command {
             Command::Rename => cmd::rename::run(argv),
             Command::Replace => cmd::replace::run(argv),
             Command::Reverse => cmd::reverse::run(argv),
+            Command::Safenames => cmd::safenames::run(argv),
             Command::Sample => cmd::sample::run(argv),
             Command::Schema => cmd::schema::run(argv),
             Command::Search => cmd::search::run(argv),
@@ -301,74 +268,13 @@ impl Command {
             Command::Slice => cmd::slice::run(argv),
             Command::Sniff => cmd::sniff::run(argv),
             Command::Sort => cmd::sort::run(argv),
+            Command::SortCheck => cmd::sortcheck::run(argv),
             Command::Split => cmd::split::run(argv),
             Command::Stats => cmd::stats::run(argv),
             Command::Table => cmd::table::run(argv),
+            Command::Tojsonl => cmd::tojsonl::run(argv),
             Command::Transpose => cmd::transpose::run(argv),
             Command::Validate => cmd::validate::run(argv),
         }
     }
 }
-
-pub type CliResult<T> = Result<T, CliError>;
-
-#[derive(Debug)]
-pub enum CliError {
-    Flag(docopt::Error),
-    Csv(csv::Error),
-    Io(io::Error),
-    Other(String),
-}
-
-impl fmt::Display for CliError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match *self {
-            CliError::Flag(ref e) => e.fmt(f),
-            CliError::Csv(ref e) => e.fmt(f),
-            CliError::Io(ref e) => e.fmt(f),
-            CliError::Other(ref s) => f.write_str(&**s),
-        }
-    }
-}
-
-impl From<docopt::Error> for CliError {
-    fn from(err: docopt::Error) -> CliError {
-        CliError::Flag(err)
-    }
-}
-
-impl From<csv::Error> for CliError {
-    fn from(err: csv::Error) -> CliError {
-        if !err.is_io_error() {
-            return CliError::Csv(err);
-        }
-        match err.into_kind() {
-            csv::ErrorKind::Io(v) => From::from(v),
-            _ => unreachable!(),
-        }
-    }
-}
-
-impl From<io::Error> for CliError {
-    fn from(err: io::Error) -> CliError {
-        CliError::Io(err)
-    }
-}
-
-impl From<String> for CliError {
-    fn from(err: String) -> CliError {
-        CliError::Other(err)
-    }
-}
-
-impl<'a> From<&'a str> for CliError {
-    fn from(err: &'a str) -> CliError {
-        CliError::Other(err.to_owned())
-    }
-}
-
-impl From<regex::Error> for CliError {
-    fn from(err: regex::Error) -> CliError {
-        CliError::Other(format!("{err:?}"))
-    }
-}
